package RPM::Source::Convert::ALTLinuxBackport 0.002;
use strict;
use warnings;
use Carp;
use Getopt::Long;
use RPM::Vercmp;
use RPM::Source::Transformation::Factory;
use RPM::Source::Convert;
use RPM::Source::Transformation::Factory::RaiseRelease;
use RPM::Source::Convert::Plugins::DistroMap::Groups;
use RPM::Source::Convert::Plugins::DistroMap::Dependencies;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

$DistroMap::enable_generic_stem_mapper=0;
$RPM::Source::Transformation::Factory::RaiseRelease::nextrel='altbackport';
$RPM::Source::Transformation::Factory::RaiseRelease::defaultchangelog='- backport';
$RPM::Source::Transformation::Factory::RaiseRelease::force_transform_mode=1;
&RPM::Source::Transformation::Factory::RaiseRelease::register_nextrel_method(
    -name => 'altbackport',
    -nextrel_method => \&nextrel_altbackport,
    -initrel_method => \&nextrel_altbackport_init,
    );

$DistroMap::default_originrepo='altlinux';
$DistroMap::default_originbranch='sisyphus';
$DistroMap::default_destrepo='altlinux';
$DistroMap::default_destbranch='p8';

our $backport_suffix;
our $verbose=0;

our @LONGOPT=(
    "A=s"   => \&__set_options,
    "B=s"   => \&__set_options,
    "M=s"   => \&__set_options,
    );

sub __set_options {
    my ($bpclass,$suf)=@_;
    $backport_suffix=join('',$bpclass,$suf);
    my $destbranch=&_suffix2destbranch($backport_suffix);
    $DistroMap::default_destbranch=$destbranch;
    print STDERR "backport: from command line: set destbranch=$destbranch suffix=$backport_suffix\n" if $verbose or $Source::Shared::verbose>1;
}

sub print_longopt {
    my ($self)=@_;
    print "ALT Linux backport specific options:\n";
    if ($Getopt::Long::bundling) {
	print "	-Mxx -- branch to backport. For example, -M70P.\n";
	print "	-Axx -- branch to autoport. For example, -A70T.\n";
    } else {
	print "	-M xx -- branch to backport. For example, -M 70P.\n";
	print "	-A xx -- branch to autoport. For example, -A 70T.\n";
    }
}

sub before_process_options {
    if ($backport_suffix) {
	$DistroMap::default_destbranch=&_suffix2destbranch($backport_suffix);
	print STDERR "backport: before_process_options: set destbranch=$DistroMap::default_destbranch from $backport_suffix\n" if $verbose;
    } elsif ($verbose) {
	print STDERR "backport: before_process_options: destbranch=$DistroMap::default_destbranch suffix=undef\n";
    }
}
sub process_options {
    unless ($backport_suffix) {
	$backport_suffix=&_destbranch2suffix($DistroMap::default_destbranch);
	print STDERR "backport: process_options: set backport_suffix=$backport_suffix from $DistroMap::default_destbranch\n" if $verbose;
    } elsif ($verbose) {
	print STDERR "backport: process_options: destbranch=$DistroMap::default_destbranch suffix=$backport_suffix\n" if $verbose;
    }
};

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'ALTLinux Backport',
	    -opt_handler_sub=>\&opt_handler_altlinux_generic_backport);
}

sub opt_handler_altlinux_generic_backport {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my ($macro_unitdir,$rpm_macros_compat);
    my $destbranch=&_suffix2destbranch($backport_suffix);
    my $mainsec=$spec->main_section;
    $mainsec->map_body(sub {
	s,^(\s*\%def_with)out(\s+backport\s*)$,${1}${2},;
	if (m,^\s*\%def_without\s+backport(\S+)\s*$, and &__branch_cmp($destbranch,$1)<=0) {
	    s,^(\s*\%def_with)out(\s+backport\S+\s*)$,${1}${2},;
	}
	s,^(\s*\%def_)dis(able\s+backport\s*)$,${1}en${2},;
	if (m,^\s*\%def_disable\s+backport(\S+)\s*$, and &__branch_cmp($destbranch,$1)<=0) {
	    s,^(\s*\%def_)dis(able\s+backport\S+\s*)$,${1}en${2},;
	}
		       });
    my @sections=$spec->get_sections();
    foreach my $sec (@sections) {
	$sec->map_body(sub {
	    # %_unitdir
	    if (/(?<!\%)\%\{?_unitdir/) {
		$macro_unitdir=1;
		if (&__branch_cmp($destbranch,'6')<0) {
		    s|(?<!\%)\%\{?_unitdir\}?|/lib/systemd/system|g;
		} elsif (&__branch_cmp($destbranch,'7')<0) {
		    s|(?<!\%)(\%\{?)(_unitdir\}?)|${1}systemd$2|g;
		}
	    }
	});
    }
    # not in rpm-build-compat :(
    #$rpm_macros_compat=1 if $macro_unitdir and &__branch_cmp($destbranch,'p7')<0;
    $mainsec->unshift_body('BuildRequires: rpm-build-compat'."\n") if $rpm_macros_compat;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __get_backport_suffix() {
    my ($self)=@_;
    return $backport_suffix;
}

sub nextrel_altbackport {
    my ($rel,$spec,undef)=@_;
    # $rel!~/\%{?release}?/; otherwise it is recurrent release invocation
    $rel=$spec->macros()->macro_subst($rel) if $rel=~/\%/ and $rel!~/\%\{?release\}?/;
    # $rel=~s/\%branch_release\s*//;
    # need after macro_subst
    $rel=~s/^\s*//;
    return &_nextrel_altbackport_raw($rel, $backport_suffix);
}

sub nextrel_altbackport_init {
    my ($spec,undef)=@_;
    my $rel=$spec->main_section->get_tag('Release');
    # need after macro_subst
    $rel=~s/^\s*//;
    return &_nextrel_altbackport_raw($rel, $backport_suffix);
}

sub _nextrel_altbackport_raw {
    my ($rel, $backportsuffix)=@_;
    die "bad Release: [$rel] (does not start with alt - not an ALT Linux release !)" unless $rel=~/^(?:alt|\%)/;
    die "bad Release: [$rel] (does not have a digit - not an ALT Linux release !)" unless $rel=~/^(?:alt.*\d|\%).*$/;
    confess "Oops: nextrel_altbackport: no backportsuffix" unless $backportsuffix;
    my $newrel = 'alt0.'.$backportsuffix.'.1';
    my ($relmajor,$relminor);
    if($rel=~/^alt(.+)\.([AM][45678][0123][CPT]?)\.(\d+)$/) {
	# already have some backport suffix
	$relmajor=$1;
	my $oldsuffix=$2;
	$relminor=$3;
	print STDERR "DEBUG:_nextrel_altbackport_raw: $rel matches some backport suffix $relmajor $oldsuffix $relminor\n" if $verbose > 2;
	my $cmp=RPM::Vercmp::evrcmp($oldsuffix,$backportsuffix);
	#print STDERR "evrcmp $oldsuffix,$backportsuffix=$cmp\n";
	if ($cmp >0 and $relmajor=~/(.*?)(\d+)$/) {
	    $relmajor=$1.($2+1);
	    $newrel = 'alt'.$relmajor.'.'.$backportsuffix.'.'.$relminor;
	} elsif ($cmp < 0) {
	    $newrel = 'alt'.$relmajor.'.'.$backportsuffix.'.'.$relminor;
	} else {
	    $newrel = 'alt'.$relmajor.'.'.$backportsuffix.'.'.($relminor+1);
	}
	return $newrel;
    } elsif ($rel=~/^alt(.+\.|)(\d+)$/) {
	$relmajor=$1;
	$relminor=$2;
	print STDERR "DEBUG:_nextrel_altbackport_raw: $rel matches case2: $relmajor $relminor\n" if $verbose > 2;
    } elsif($rel=~/^alt(.+)(\d+)$/) {
	$relmajor=$1;
	$relminor=$2;
	print STDERR "DEBUG:_nextrel_altbackport_raw: $rel matches case3: $relmajor $relminor\n" if $verbose > 2;
    }
    if ($relminor) {
	$newrel = 'alt'.$relmajor.($relminor-1).'.'.$backportsuffix.'.1';
    }
    return $newrel;
}

sub _suffix2destbranch {
    my ($suffix)=@_;
    my $destbranch='p8';
    unless ($suffix=~/^([ABM])(.+)$/) {
	print STDERR "WARNING: unsupported suffix $suffix\n";
	return $destbranch;
    }
    my ($bpclass,$suf)=($1,$2);
    if (($bpclass eq 'A' or $bpclass eq 'B') and $suf =~ '^([6789])0$') {
	$destbranch='t'.$1;
    } elsif ($suf =~ '^([56789])0([CPT])$') {
	$destbranch="\L$2\E${1}";
    } elsif ($suf =~ '^([5432])([012])$') {
	$destbranch="$1.$2";
    } else {
	print STDERR "WARNING: unsupported suffix $suffix\n";
    }
    return $destbranch;
}

sub _destbranch2suffix {
    my ($branch)=@_;
    my $suffix='M80P';
    if ($branch =~ '^([cpt])([56789])$') {
	$suffix="M${2}0\U$1\E";
    } elsif ($branch =~ '^([5432])\.([012])$') {
	$suffix="M$1$2";
    } else {
	print STDERR "WARNING: unsupported branch $branch\n";
    }
    return $suffix;
}

sub __branch_cmp {
    my ($branch1, $branch2)=@_;
    $branch1=~s/^([cpt])([\d\.]+)/$2$1/;
    $branch2=~s/^([cpt])([\d\.]+)/$2$1/;
    return RPM::Vercmp::vercmp($branch1,'',$branch2,'');
}

1;
