package RPM::Source::Convert::Plugins::DistroMap::Dependencies;
use strict;
use warnings;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory::DependencyFilter;

use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

sub register_messages {
    my ($self,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-distromap_map_provides		=> 0,
	-distromap_map_obsoletes	=> 0,
	-distromap_map_conflicts	=> 1,
    });
}

# 0 transformations; for register_messages only
sub transformations {
    return;
}

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'distromap_binary_flags',
    -description => 'filter that implements DistroMap binary flags',
    -class => 'RPM::Source::Tools::Dependency::Filter::DistroMapBinaryFlags',
    -priority => FILTER_PRIORITY_DISTROMAP,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'distromap_binary_names',
    -description => 'filter that implements DistroMap binary package name substitutions',
    -class => 'RPM::Source::Tools::Dependency::Filter::DistroMap',
    -priority => FILTER_PRIORITY_DISTROMAP,
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS | SPEC_TAG_PROVIDES | SPEC_TAG_OBSOLETES,
    -breaks_tokenization => 1,
);

package RPM::Source::Tools::Dependency::Filter::DistroMapBinaryFlags;

use strict;
use warnings;
use Source::Shared::Resource::DistroMap;

our @ISA=q/RPM::Source::Tools::Dependency::Filter/;
our $verbose=0;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $version=$dep->{-version};
    my $distromap=$self->{-resourceobj}->get_distromap;
    my $expanded_name=$self->{-spec}->macros()->macro_subst($dep->{-name});
    if ($distromap->flag('flags/binary/exclude',$expanded_name)) {
	print STDERR "info: flags/binary/exclude: dropped $expanded_name dependency\n" if $verbose;
	delete $dep->{-name};
	return;
    }
    return $dep unless defined $version;
    if ($distromap->flag('flags/binary/noversion',$expanded_name)) {
	print STDERR "info: flags/binary/noversion: dropped $dep->{-version} in $expanded_name $dep->{-relation} $dep->{-version}\n" if $verbose;
	delete $dep->{-version};
	delete $dep->{-relation};
	return $dep;
    }
    if ($version=~/^\d+:/ and $distromap->flag('flags/binary/noepoch',$expanded_name)) {
	print STDERR "info: flags/binary/noepoch: dropped $dep->{-version} in $expanded_name $dep->{-relation} $dep->{-version}\n" if $verbose;
	$dep->{-version}=~s/^\d+://;
    }
    return $dep;
}

1;

package RPM::Source::Tools::Dependency::Filter::DistroMap;
use strict;
use warnings;
use RPM::Source::Tools::Dependency::Filter;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;
our $verbose=1;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $opt=$self->{-opt};
    my $tagbits=$tagobj->tagbits;
    if ($verbose>2) {
	warn "DEBUG: DistroMap::Dependencies: ignored Provides (no opt)" if ($tagbits & SPEC_TAG_PROVIDES) and not $opt->{-distromap_map_provides};
	warn "DEBUG: DistroMap::Dependencies: ignored Obsoletes (no opt)" if ($tagbits & SPEC_TAG_OBSOLETES) and not $opt->{-distromap_map_obsoletes};
	warn "DEBUG: DistroMap::Dependencies: ignored Conflicts (no opt)" if ($tagbits & SPEC_TAG_SET_CONFLICTS) and not $opt->{-distromap_map_conflicts};
    }
    return $dep if ($tagbits & SPEC_TAG_SET_CONFLICTS) and not $opt->{-distromap_map_conflicts};
    return $dep if ($tagbits & SPEC_TAG_OBSOLETES) and not $opt->{-distromap_map_obsoletes};
    return $dep if ($tagbits & SPEC_TAG_PROVIDES) and not $opt->{-distromap_map_provides};

    my $resourceobj=$self->{-resourceobj};
    return $dep unless $resourceobj;
    my $distromap=$self->{-resourceobj}->get_distromap;
    return $dep unless $distromap;
    my $spec=$self->{-spec};
    my $pkgname=$dep->{-name};

    # do nothing; %{name} will change authomatically
    if ($pkgname eq '%{name}' or $pkgname eq '%name') {
    # filename dependencies that are currenly not in apt index
    } elsif ($pkgname=~m!^/usr/include/!) {
	my $header=$pkgname;
	$header=~s,^/usr/include/,,;
	my @mapping=$distromap->dest()->translate('headers',$header);
	$dep->{-name}=$mapping[0] if @mapping and (@mapping ==1);
    } else {
	# like, 
	# %define req_list dep0, dep1, ...
        # Requires: %{req_list}
	# TODO: handle %(eval ...) above apply()
	my @expanded_names=split(/\s+/,$spec->macros()->macro_subst_if_defined($pkgname));
	my @mappings;
	foreach my $expanded_name (@expanded_names) {
	    my @mapping=$distromap->multimap_binary_origin2dest($expanded_name);
	    if ($Source::Shared::verbose>1) {
		my $mapname=join(' ',@mapping);
		my $originname="$expanded_name [$pkgname]";
		$originname=$pkgname if $expanded_name eq $pkgname;
		print STDERR "distromap: $originname -> $mapname\n" if $mapname;
		print STDERR "distromap: no map for $originname\n" unless $mapname;
	    }
	    if (@mapping) {
		push @mappings, @mapping;
	    } else {
		push @mappings, $expanded_name;
	    }
	}
	warn "Oops!Dependencies.pm: undefined mappings[0] for $pkgname, expanded_names=".join(' ',@expanded_names) if $verbose and not defined $mappings[0] and @expanded_names>1;
	# regen deps objects
	if (@mappings and 
	    (($mappings[0] ne $expanded_names[0]) or 
	     # append; like boost-devel unpack to boost-devel + boost-*-devel
	     (scalar @mappings ne scalar @expanded_names))) {
	    if (@mappings==1) {
		$dep->{-name}=$mappings[0];
	    } elsif (@mappings >1) {
		return map {RPM::Source::Tools::Dependency->new(-name=>$_)} @mappings;
	    }
	}
    }
    return $dep;
}

1;
