package RPM::Source::Convert::Plugins::DistroMap::Groups;
use strict;
use warnings;
use Carp;

use Source::Shared::Resource::DistroMap;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose;

our %ALLOWED_GROUP;
# generic conversions
our %EXTRA_GROUP_POSTMAP = ();
# specific conversions
our %EXTRA_GROUP_PREMAP = ();

our $default_group='Other';
# TODO
my $opt_default_group;

sub print_longopt {
    print "DistroMap::Groups options:\n";
    print "	--default-group <default group>. Default is $default_group.\n";
}

our @LONGOPT=(
    'def-group|default-group=s' => \$default_group,
    );

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'distromap_group_convert',
	    -opt_handler_sub=>\&opt_handler_distromap_group_convert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRERELEASE,
	    -name => 'distromap_group_posthook',
	    -opt_handler_sub=>\&opt_handler_distromap_group_posthook);
}

sub init_factory {
    my ($class,$resourceobj)=@_;
    return if %ALLOWED_GROUP;

    # chroot build; no distromap
    unless ($ENV{'RPM_BUILD_ROOT'}) {
	my $distromap=$resourceobj->get_distromap;
	if ($distromap) {
	    # TODO
	    my $dest=$distromap->dest;
	    # TODO: need an interface to load a list
	    # 'groups-allowed'
	}
	return if %ALLOWED_GROUP;
    }

    if (-e '/usr/lib/rpm/GROUPS') {
	open my $rpmgroup, '<' , '/usr/lib/rpm/GROUPS' || die "can't open /usr/lib/rpm/GROUPS: $!";
	my $group;
	while ($group=<$rpmgroup>) {
	    chomp $group;
	    $group=~s/\s*$//;
	    $ALLOWED_GROUP{$group}=1 if $group;
	}
	close $rpmgroup;
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

# TODO: process_options callback sub? or init will do?
#$opt_default_group||=$default_group

sub opt_handler_distromap_group_posthook {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $mainsec=$spec->main_section;

    # take a missing group from parent
    if ($parent) {
	foreach my $sec ($spec->get_sections()) {
	    if ($sec->get_type eq 'package' and not $sec->get_tag('Group')) {
		my $p = $sec->get_raw_package();
		my $parentsec=$parent->get_section('package',$p);
		next unless $parentsec;
		my $parentgroup=$parentsec->get_tag('Group');
		next unless $parentgroup;
		$sec->set_tag('Group',$parentgroup);
	    }
	}
    }

    unless (defined $mainsec->get_tag('Group')) {
	my $new_group=$default_group;
	&__guess_group_from_pkgname($mainsec, \$new_group) if not $new_group or $new_group eq 'Other';
	$new_group||='Other';
	warn "WARNING: main section does not contain a Group: tag. Using $new_group\n";
	$mainsec->set_tag('Group',$new_group);
    }

    my $distromap=$resourceobj->get_distromap;
    foreach my $sec ($spec->get_sections()) {
	&distromap_group_postmap($distromap,$sec,$spec,$parent) if $sec->get_type eq 'package';
    }

    my $maingroup=$spec->main_section->get_tag('Group');
    # generic_merge_main_group_if_group_is_missing to avoid
    # error: Group field must be present in package: <subpackage>
    foreach my $sec ($spec->get_sections()) {
	$sec->set_tag('Group',$maingroup) if $sec->get_type eq 'package' and not $sec->get_tag('Group');
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_distromap_group_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $distromap=$resourceobj->get_distromap;
    foreach my $sec ($spec->get_sections()) {
	&distromap_group_premap($distromap,$spec,$sec) if $sec->get_type eq 'package';
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub distromap_group_premap {
    my ($distromap,$spec,$section)=@_;
    my @out;
    foreach (@{$section->get_bodyref()}) {
	if (/^Group(\s*):(\s*)(\S.*\S)\s*$/i) {
	    my $gprespace=$1;
	    my $gpostspace=$2;
	    my $group=$3;
	    my $group_premap=$distromap->unimap_exact_group_origin2dest($group);
	    $group_premap=$EXTRA_GROUP_PREMAP{$group} unless $group_premap;
	    if ($group_premap) {
		push @out, "Group$gprespace:$gpostspace$group_premap\n";
	    } else {
		push @out, $_;
	    }
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

sub distromap_group_postmap {
    my ($distromap,$section,$spec,$parent)=@_;
    my @out;
    foreach (@{$section->get_bodyref()}) {
	if (/^Group(\s*):(\s*)(\S.*\S)\s*$/i) {
	    my $gprespace=$1;
	    my $gpostspace=$2;
	    my $group=$3;
	    my $newgroup;
	    if ($group=~/^\%/) {
		$group=$spec->macros->macro_subst($group);
	    }
	    unless ($ALLOWED_GROUP{$group} or $group eq 'Other') {
		if ($parent) {
		    my $parentsec=$parent->get_section('package',$section->get_raw_package());
		    $newgroup||=$parentsec->get_tag('Group') if $parentsec;
		}
		unless ($newgroup) {
		    &__guess_group_from_pkgname($section, \$newgroup);
		}
		unless ($newgroup) {
		    my $group_postmap=$distromap->unimap_approximate_group_origin2dest($group);
		    $group_postmap=$EXTRA_GROUP_POSTMAP{$group} unless $group_postmap;
		    print STDERR "warning: substituting Group: $group with unaccurate $group_postmap\n" if $group_postmap;
		    $newgroup||=$group_postmap;
		}
		unless ($newgroup) {
		    my $pkg=$section->get_raw_package();
		    $pkg||='main section';
		    print STDERR "WARNING: forbidden Group: $group in $pkg. Please, replace.\n";
		}
	    }

	    if ($newgroup) {
		push @out, "Group$gprespace:$gpostspace$newgroup\n";
	    } else {
		push @out, $_;
	    }
	} else {
	    push @out, $_;
	}
    }    
    $section->set_body(\@out);
}


sub __guess_group_from_pkgname {
    my ($section, $newgroupref)=@_;
    my $sec_pkg_expanded = $section->get_package_name();
    if ($sec_pkg_expanded =~/^python\d?-module/) {
	&__try_group($newgroupref,'Development/Python');
    } elsif ($sec_pkg_expanded =~/^perl-/) {
	&__try_group($newgroupref,'Development/Perl');
    } elsif ($sec_pkg_expanded =~/^nodejs-/) {
	&__try_group($newgroupref,'Development/Other');
    } elsif ($sec_pkg_expanded =~/^golang-/) {
	&__try_group($newgroupref,'Development/Other');
    } elsif ($sec_pkg_expanded =~/^fonts-[ot]tf-/) {
	&__try_group($newgroupref,'System/Fonts/True type');
    } elsif ($sec_pkg_expanded =~/^fonts-type1-/) {
	&__try_group($newgroupref,'System/Fonts/Type1');
    } elsif ($sec_pkg_expanded =~/^fonts-bitmap-/) {
	&__try_group($newgroupref,'System/Fonts/X11 bitmap');
    } elsif ($sec_pkg_expanded =~/^firmware-/ or $sec_pkg_expanded =~/-firmware$/) {
	&__try_group($newgroupref,'System/Kernel and hardware');
    }
}

sub __try_group {
    my ($ptr, $name)=@_;
    $$ptr=$name if $ALLOWED_GROUP{$name};
}

1;
