package RPM::Source::Convert::Plugins::FedoraFontPkg2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Tools::Dependency::Filter;
use Source::Shared::Resource::DistroMap;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Factory::DependencyFilter;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

# fake section to be processed in &convert_font_pkg
# a method to protect multisection macro from killing as the part of %clean section
use RPM::Source::SpecSection;
$RPM::Source::SpecSection::SECTION_MACROS{'_font_pkg'}=1;

our $verbose=0;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Fedora Font dependencies',
    -description => 'convert Fedora specific Font dependencies',
    -class => 'RPM::Source::Tools::Dependency::Filter::FedoraFont',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS | SPEC_TAG_PROVIDES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-enable_fedora_font_posthook => 0,
	});
}

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'fedora2alt_preconvert_font_pkg',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_font_pkg_preconvert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_POSTHOOKS,
	    # TODO: insert_first => 1 ? it seems there is no need
	    -name => 'fedora2alt_posthook_font_pkg',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_font_pkg_posthook);
}

sub opt_handler_fedora2alt_font_pkg_posthook {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK
	if not $opt->{-enable_fedora_font_posthook};

    print STDERR "opt_handler_fedora2alt_font_pkg_posthook: start\n" if $verbose>1;

    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    foreach my $sec (@sections) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	} elsif ($t eq 'build') {
	} elsif ($t eq 'install') {
	    if (not $spec->{'CONVERT::FEDORA::NO_MOVE_FONTS'}) {
		$sec->push_body(q!# generic fedora font import transformations
# move fonts to corresponding subdirs if any
for fontpatt in OTF TTF TTC otf ttf ttc pcf pcf.gz bdf afm pfa pfb; do
    case "$fontpatt" in 
	pcf*|bdf*) type=bitmap;;
	tt*|TT*) type=ttf;;
	otf|OTF) type=otf;;
	afm*|pf*) type=type1;;
    esac
    find $RPM_BUILD_ROOT/usr/share/fonts -type f -name '*.'$fontpatt | while read i; do
	j=`echo "$i" | sed -e s,/usr/share/fonts/,/usr/share/fonts/$type/,`;
	install -Dm644 "$i" "$j";
	rm -f "$i";
	olddir=`dirname "$i"`;
	mv -f "$olddir"/{encodings.dir,fonts.{dir,scale,alias}} `dirname "$j"`/ 2>/dev/null ||:
	rmdir -p "$olddir" 2>/dev/null ||:
    done
done
!);
	    }
	    if (not $spec->{'CONVERT::FEDORA::NO_RELINK_FONTCATALOG'}) {
		$sec->push_body(q!# kill invalid catalogue links
if [ -d $RPM_BUILD_ROOT/etc/X11/fontpath.d ]; then
    find -L $RPM_BUILD_ROOT/etc/X11/fontpath.d -type l -print -delete ||:
    # relink catalogue
    find $RPM_BUILD_ROOT/usr/share/fonts -name fonts.dir | while read i; do
	pri=10;
	j=`echo $i | sed -e s,$RPM_BUILD_ROOT/usr/share/fonts/,,`; type=${j%%%%/*}; 
	pre_stem=${j##$type/}; stem=`dirname $pre_stem|sed -e s,/,-,g`;
	case "$type" in 
	    bitmap) pri=10;;
	    ttf|ttf) pri=50;;
	    type1) pri=40;;
	esac
	ln -s /usr/share/fonts/$j $RPM_BUILD_ROOT/etc/X11/fontpath.d/"$stem:pri=$pri"
    done ||:
fi
!);
	    }
	} elsif ($t eq 'files') {
	    $sec->subst_body(qr'\%\{_fontdir\}','%{_fontbasedir}/*/%{_fontstem}');
	    $sec->map_body(sub {
		if (m!^(.*)\%\{catalogue\}/(\S+)\s*$!) {
		    my $pathbase=$1;
		    my $stem=$2;
		    $stem=~s/-fonts$//;
		    $_=$pathbase.'%{catalogue}/'.$stem."*\n" unless /\*\s*$/;
		}
	    });
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_fedora2alt_font_pkg_preconvert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $distromap=$resourceobj->get_distromap;
    # first search for %_font_pkg
    my $flag_font_pkg_processed=0;
    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    unless ($mainsec->match_body(qr'Build(?:Pre)?Req.*fontpackages-devel')) {
	print STDERR "DEBUG: fedora2alt_font_pkg_preconvert: fontpackages-devel not found, exiting...\n" if $verbose>2;
	return RPM::Source::TransformContainer::TRANSFORMATION_OK;
    }

    my $source_name=$spec->macros()->macro_subst('%name');
    my $font_type=&__get_font_type_from_name('',$distromap->unimap_source_origin2dest($source_name));
    print STDERR "DEBUG: fedora2alt_font_pkg_preconvert:$source_name: font type=$font_type.\n" if $verbose>2;

    foreach my $sec (@sections) {
	$sec->apply_body_and_header(sub {
	    local $_=shift;
	    unless (/^\s*\%\{?_font_pkg\}?/) {
		#print STDERR "DEBUG: fedora2alt_font_pkg_preconvert not found in: $_" if $verbose>8;
		return $_;
	    } else {
		print STDERR "DEBUG: fedora2alt_font_pkg_preconvert activated: \%_font_pkg detected: $_" if $verbose>1;
		$flag_font_pkg_processed=1;
		chomp;
		s/^\s*\%\{?_font_pkg\}?\s*//;
		my @out='%files'."\n";
		@out='%files -n fonts-ttf-unifont'."\n" if $source_name eq 'unifont';
		my @filepattern;
		while (s/^(\S+)\s*//) {
		    if ($1 eq '-f') {
			s/^(\S+)\s*//;
			push @out, '%{_fontconfig_templatedir}/'."$1\n";
			push @out, '%config(noreplace) %{_fontconfig_confdir}/'."$1\n";
		    } elsif ($1 eq '-n') {
			s/^(\S+)\s*//;
			my $fontstem=$1;
			my $fcname=lc($fontstem); # in fc they lc :) see mathjax
			my $pkgsec=$spec->get_section('package',"-n $fcname");
			if ($pkgsec) {
			    print STDERR "fedora2alt_preconvert_font_pkg: found %package -n $fcname\n" if $verbose>1;
			} else {
			    print STDERR "fedora2alt_preconvert_font_pkg: not found %package -n $fcname\n" if $verbose>1;
			    $pkgsec=$spec->get_section('package',"-n $fcname-fonts");
			    if ($pkgsec) {
				$fcname .= '-fonts';
				print STDERR "fedora2alt_preconvert_font_pkg: found %package -n $fcname-fonts\n" if $verbose>1;
			    } else {
				print STDERR "fedora2alt_preconvert_font_pkg: not found %package -n $fcname-fonts\n" if $verbose>1;
				$pkgsec=$spec->get_section('package',"$fcname-fonts");
				if ($pkgsec) {
				    print STDERR "fedora2alt_preconvert_font_pkg: found %package $fcname-fonts\n" if $verbose>1;
				    $fcname = $spec->macros->macro_subst('%name').'-'.$fcname.'-fonts';
				}
			    }
			}
			# -n can be a real package name as in ht-alegreya-fonts
			if (not $pkgsec) {
			    $fcname='%{?fontname:%fontname}%{!?fontname:%name}-'.$fontstem.'-fonts';
			    print STDERR "fedora2alt_preconvert_font_pkg: fontstem=$fontstem fcname=$fcname\n" if $verbose>1;
			    # mplus-fonts
			    $fcname=$fontstem.'-fonts' if ($fontstem =~/fontname/ and $fcname!~/-fonts$/);
			}
			my @mapname=$distromap->multimap_binary_origin2dest($spec->macros()->macro_subst($fcname));
			$font_type=&__get_font_type_from_name($font_type,@mapname);
			$mapname[0]||=$fcname;
			$out[0] ='%files -n '.$mapname[0]."\n";
		    } else {
			push @filepattern, $1;
		    }
		}
		s/\s*$//;
		push @filepattern, $_ if $_;
		# does not work for multi packages
		#push @out, '%{_fontbasedir}/*'."\n";
		push @out,
		    # or else dir will be unowned
		    '%dir %{_fontbasedir}/*/%{_fontstem}/'."\n",
			map { '%{_fontbasedir}/*/%{_fontstem}/'.$_."\n"} 
			map {$_ eq '%{_fontdir}' ? '*': $_} @filepattern;
		return @out;
	    }
			 }, '%_font_pkg converter');
    }
    # not really needed; fontdir (not _fontdir) is used in few packages? baekamuk-bdf-fonts
    #$sections->push_body('%define fontdir %{_datadir}/fonts/'.$font_type.'/%{_fontstem}'."\n") if $font_type;

    if ($flag_font_pkg_processed) {
        print STDERR "opt_handler_fedora2alt_font_pkg_preconvert: -enable_fedora_font_posthook => 1\n" if $verbose>1;
	$spec->_reset_speclist() ;
	$opt->{-enable_fedora_font_posthook}=1;
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __get_font_type_from_name {
    my ($font_type,@mapname)=@_;
    if (@mapname and $mapname[0] =~ /^fonts-(ttf|otf|type1|bitmap)-/) {
	if ($font_type) {
	    $font_type='default' if $font_type ne $1;
	} else {
	    $font_type=$1;
	}
    }
    return $font_type;
}

1;

package RPM::Source::Tools::Dependency::Filter::FedoraFont;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    return $dep if $pkgname!~/^font\(:lang=(.+)\)$/;
    my $langreq=$1;
    $dep->{-name}='fontlang('.$langreq.')';
    return $dep;
}

1;

__END__
